@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2018, DIANA-HEP
+# Copyright (c) 2019, IRIS-HEP
 # All rights reserved.
 # 
 # Redistribution and use in source and binary forms, with or without
@@ -57,7 +57,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
 
     def deepcopy(self, chunks=None, counts=None):
         out = self.copy(chunks=chunks, counts=counts)
-        out._chunks = [awkward.util.deepcopy(out._chunks) for x in out._chunks]
+        out._chunks = [self._util_deepcopy(out._chunks) for x in out._chunks]
         return out
 
     def _mine(self, overrides):
@@ -67,26 +67,26 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
         self.knowcounts()
         self._valid()
         mine = self._mine(overrides)
-        return self.copy([awkward.util.numpy.empty_like(x) if isinstance(x, awkward.util.numpy.ndarray) else x.empty_like(**overrides) for x in self._chunks], counts=list(self._counts), **mine)
+        return self.copy([self.numpy.empty_like(x) if isinstance(x, self.numpy.ndarray) else x.empty_like(**overrides) for x in self._chunks], counts=list(self._counts), **mine)
 
     def zeros_like(self, **overrides):
         self.knowcounts()
         self._valid()
         mine = self._mine(overrides)
-        return self.copy([awkward.util.numpy.zeros_like(x) if isinstance(x, awkward.util.numpy.ndarray) else x.zeros_like(**overrides) for x in self._chunks], counts=list(self._counts), **mine)
+        return self.copy([self.numpy.zeros_like(x) if isinstance(x, self.numpy.ndarray) else x.zeros_like(**overrides) for x in self._chunks], counts=list(self._counts), **mine)
 
     def ones_like(self, **overrides):
         self.knowcounts()
         self._valid()
         mine = self._mine(overrides)
-        return self.copy([awkward.util.numpy.ones_like(x) if isinstance(x, awkward.util.numpy.ndarray) else x.ones_like(**overrides) for x in self._chunks], counts=list(self._counts), **mine)
+        return self.copy([self.numpy.ones_like(x) if isinstance(x, self.numpy.ndarray) else x.ones_like(**overrides) for x in self._chunks], counts=list(self._counts), **mine)
 
     def __awkward_persist__(self, ident, fill, prefix, suffix, schemasuffix, storage, compression, **kwargs):
         self.knowcounts()
         self._valid()
         return {"id": ident,
-                "call": ["awkward", self.__class__.__name__],
-                "args": [{"list": [fill(x, self.__class__.__name__ + ".chunk", prefix, suffix, schemasuffix, storage, compression, **kwargs) for c, x in zip(self._counts, self._chunks) if c > 0]},
+                "call": ["awkward", "ChunkedArray"],
+                "args": [{"list": [fill(x, "ChunkedArray.chunk", prefix, suffix, schemasuffix, storage, compression, **kwargs) for c, x in zip(self._counts, self._chunks) if c > 0]},
                          {"json": [int(c) for c in self._counts if c > 0]}]}
 
     @property
@@ -99,7 +99,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
             iter(value)
         except TypeError:
             raise TypeError("chunks must be iterable")
-        self._chunks = [awkward.util.toarray(x, self.DEFAULTTYPE) for x in value]
+        self._chunks = [self._util_toarray(x, self.DEFAULTTYPE) for x in value]
         self._types = [None] * len(self._chunks)
 
     @property
@@ -109,7 +109,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
     @counts.setter
     def counts(self, value):
         try:
-            if not all(isinstance(x, awkward.util.integer) and x >= 0 for x in value):
+            if not all(self._util_isinteger(x) and x >= 0 for x in value):
                 raise ValueError("counts must contain only non-negative integers")
         except TypeError:
             raise TypeError("counts must be iterable")
@@ -118,9 +118,8 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
 
     @property
     def offsets(self):
-        import awkward.array.jagged
         if self._offsets is None or len(self._offsets) != len(self._counts) + 1:
-            self._offsets = awkward.array.jagged.counts2offsets(self._counts)
+            self._offsets = self.JaggedArray.counts2offsets(self._counts)
         return self._offsets
 
     @property
@@ -152,7 +151,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
     def global2chunkid(self, index, return_normalized=False):
         self._valid()
 
-        if isinstance(index, awkward.util.integer):
+        if self._util_isinteger(index):
             original_index = index
             if index < 0:
                 index += len(self)
@@ -167,24 +166,24 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
                 cumulative += count
                 self._counts.append(count)
 
-            out = awkward.util.numpy.searchsorted(self.offsets, index, "right") - 1
+            out = self.numpy.searchsorted(self.offsets, index, "right") - 1
 
         else:
-            index = awkward.util.numpy.array(index, copy=False)
-            if len(index.shape) == 1 and issubclass(index.dtype.type, awkward.util.numpy.integer):
+            index = self.numpy.array(index, copy=False)
+            if len(index.shape) == 1 and self._util_isintegertype(index.dtype.type):
                 if len(index) == 0:
-                    out = awkward.util.numpy.empty(0, dtype=self.INDEXTYPE)
+                    out = self.numpy.empty(0, dtype=self.INDEXTYPE)
 
                 else:
                     mask = (index < 0)
                     if mask.any():
-                        index = awkward.util.deepcopy(index)
+                        index = self._util_deepcopy(index)
                         index[mask] += len(self)
                     if (index < 0).any():
                         raise IndexError("index out of bounds for length {0}".format(len(self)))
 
                     self.global2chunkid(index.max())    # make sure all the counts we need are known
-                    out = awkward.util.numpy.searchsorted(self.offsets, index, "right") - 1
+                    out = self.numpy.searchsorted(self.offsets, index, "right") - 1
 
             else:
                 raise TypeError("global2chunkid requires an integer or an array of integers")
@@ -197,13 +196,13 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
     def global2local(self, index):
         chunkid, index = self.global2chunkid(index, return_normalized=True)
 
-        if isinstance(index, awkward.util.integer):
+        if self._util_isinteger(index):
             return self._chunks[chunkid], index - self.offsets[chunkid]
         else:
-            return awkward.util.numpy.array(self._chunks, dtype=awkward.util.numpy.object)[chunkid], index - self.offsets[chunkid]
+            return self.numpy.array(self._chunks, dtype=self.numpy.object)[chunkid], index - self.offsets[chunkid]
 
     def local2global(self, index, chunkid):
-        if isinstance(chunkid, awkward.util.integer):
+        if self._util_isinteger(chunkid):
             self.knowcounts(chunkid + 1)
             self._valid()
             original_index = index
@@ -214,15 +213,15 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
             return self.offsets[chunkid] + index
 
         else:
-            index = awkward.util.numpy.array(index, copy=False)
-            chunkid = awkward.util.numpy.array(chunkid, copy=False)
-            if len(index.shape) == 1 and issubclass(index.dtype.type, awkward.util.numpy.integer) and len(chunkid.shape) == 1 and issubclass(chunkid.dtype.type, awkward.util.numpy.integer):
+            index = self.numpy.array(index, copy=False)
+            chunkid = self.numpy.array(chunkid, copy=False)
+            if len(index.shape) == 1 and self._util_isintegertype(index.dtype.type) and len(chunkid.shape) == 1 and self._util_isintegertype(chunkid.dtype.type):
                 if len(index) != len(chunkid):
                     raise ValueError("len(index) is {0} and len(chunkid) is {1}, but they should be equal".format(len(index), len(chunkid)))
 
                 self.knowcounts(chunkid.max() + 1)
                 self._valid()
-                counts = awkward.util.numpy.array(self._counts, dtype=self.INDEXTYPE)
+                counts = self.numpy.array(self._counts, dtype=self.INDEXTYPE)
                 mask = (index < 0)
                 index[mask] += counts[mask]
                 if not ((0 <= index) & (index < counts)).all():
@@ -276,7 +275,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
         if self.countsknown:
             return super(ChunkedArray, self).__str__()
         else:
-            strs = [awkward.util.array_str(x) for x in self[:7].__iter__(checkiter=False)]
+            strs = [self._util_arraystr(x) for x in self[:7].__iter__(checkiter=False)]
             if len(strs) < 7:
                 return super(ChunkedArray, self).__str__()
             else:
@@ -294,11 +293,11 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
     def __array__(self, *args, **kwargs):
         self._checktonumpy()
 
-        if isinstance(self.type.to, awkward.util.numpy.dtype):
+        if isinstance(self.type.to, self.numpy.dtype):
             if len(self) == 0:
-                return awkward.util.numpy.empty(0, dtype=self.DEFAULTTYPE)
+                return self.numpy.empty(0, dtype=self.DEFAULTTYPE)
             else:
-                out = awkward.util.numpy.empty(self.shape, dtype=self.dtype)
+                out = self.numpy.empty(self.shape, dtype=self.dtype)
                 for chunk, slc in zip(self._chunks, self._slices()):
                     out[slc] = chunk
                 return out
@@ -306,10 +305,9 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
             return super(ChunkedArray, self).__array__(*args, **kwargs)
 
     def __getitem__(self, where):
-        import awkward.array.indexed
         self._valid()
 
-        if awkward.util.isstringslice(where):
+        if self._util_isstringslice(where):
             if isinstance(where, awkward.util.string):
                 if not self.type.hascolumn(where):
                     raise ValueError("no column named {0}".format(repr(where)))
@@ -333,7 +331,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
             where = (where,)
         head, tail = where[0], where[1:]
 
-        if isinstance(head, awkward.util.integer):
+        if self._util_isinteger(head):
             chunk, localhead = self.global2local(head)
             return chunk[(localhead,) + tail]
 
@@ -384,10 +382,10 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
                     stop = len(self)
                 if step > 0:
                     # we want the chunkid at or to the right of stop (no -1)
-                    stop_chunkid = min(awkward.util.numpy.searchsorted(self.offsets, stop, "right"), len(self._chunks))
+                    stop_chunkid = min(self.numpy.searchsorted(self.offsets, stop, "right"), len(self._chunks))
                 else:
                     # we want the chunkid to the left of stop
-                    stop_chunkid = max(awkward.util.numpy.searchsorted(self.offsets, stop, "right") - 2, -1)
+                    stop_chunkid = max(self.numpy.searchsorted(self.offsets, stop, "right") - 2, -1)
 
             offsets = self.offsets
             chunks = []
@@ -433,8 +431,8 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
             return self.copy(chunks=chunks)
 
         else:
-            head = awkward.util.numpy.array(head, copy=False)
-            if len(head.shape) == 1 and issubclass(head.dtype.type, awkward.util.numpy.integer):
+            head = self.numpy.array(head, copy=False)
+            if len(head.shape) == 1 and self._util_isintegertype(head.dtype.type):
                 if len(head) == 0 and len(self._chunks) == 0:
                     return self.copy(chunks=[])[tail]
                 elif len(head) == 0:
@@ -444,23 +442,23 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
 
                 diff = (chunkid[1:] - chunkid[:-1])
                 if (diff >= 0).all():
-                    diff2 = awkward.util.numpy.empty(len(chunkid), dtype=self.INDEXTYPE)
+                    diff2 = self.numpy.empty(len(chunkid), dtype=self.INDEXTYPE)
                     diff2[0] = 1
                     diff2[1:] = diff
                     mask = (diff2 > 0)
-                    offsets = list(awkward.util.numpy.nonzero(mask)[0]) + [len(chunkid)]
+                    offsets = list(self.numpy.nonzero(mask)[0]) + [len(chunkid)]
                     chunks = []
                     for i, cid in enumerate(chunkid[mask]):
                         localindex = head[offsets[i]:offsets[i + 1]] - self.offsets[cid]
                         chunks.append(self._chunks[cid][localindex])
                     return self.copy(chunks=chunks)
 
-                elif awkward.util.isnumpy(self.type):
-                    out = awkward.util.numpy.empty((len(head),) + self.type.shape[1:], dtype=self.type.dtype)
+                elif self._util_isnumpy(self.type):
+                    out = self.numpy.empty((len(head),) + self.type.shape[1:], dtype=self.type.dtype)
                     self.knowcounts(chunkid.max())
                     offsets = self.offsets
 
-                    for cid in awkward.util.numpy.unique(chunkid):
+                    for cid in self.numpy.unique(chunkid):
                         mask = (chunkid == cid)
                         out[mask] = self._chunks[cid][head[mask] - offsets[cid]]
 
@@ -470,12 +468,12 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
                         return out[(slice(None),) + tail]
 
                 elif tail == ():
-                    return awkward.array.indexed.IndexedArray(head, self)
+                    return self.IndexedArray(head, self)
 
                 else:
                     raise NotImplementedError
 
-            elif len(head.shape) == 1 and issubclass(head.dtype.type, (awkward.util.numpy.bool, awkward.util.numpy.bool_)):
+            elif len(head.shape) == 1 and issubclass(head.dtype.type, (self.numpy.bool, self.numpy.bool_)):
                 if len(self) != len(head):
                     raise IndexError("boolean index did not match indexed array along dimension 0; dimension is {0} but corresponding boolean dimension is {1}".format(len(self), len(head)))
 
@@ -508,7 +506,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
         if isinstance(where, awkward.util.string):
             for chunk in self._chunks:
                 del chunk[where]
-        elif awkward.util.isstringslice(where):
+        elif self._util_isstringslice(where):
             for chunk in self._chunks:
                 for x in where:
                     del chunk[x]
@@ -516,8 +514,6 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
             raise TypeError("invalid index for removing column from Table: {0}".format(where))
 
     def __array_ufunc__(self, ufunc, method, *inputs, **kwargs):
-        import awkward.array.objects
-
         if "out" in kwargs:
             raise NotImplementedError("in-place operations not supported")
 
@@ -546,7 +542,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
             for x in inputs:
                 if isinstance(x, ChunkedArray):
                     batch.append(x._chunks[i])
-                elif isinstance(x, (awkward.util.numpy.ndarray, awkward.array.base.AwkwardArray)):
+                elif isinstance(x, (self.numpy.ndarray, awkward.array.base.AwkwardArray)):
                     batch.append(x[slc])
                 else:
                     batch.append(x)
@@ -562,7 +558,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
                 if out is None:
                     out = list(result)
                 for i, x in enumerate(result):
-                    if isinstance(x, (awkward.util.numpy.ndarray, awkward.array.base.AwkwardArray)):
+                    if isinstance(x, (self.numpy.ndarray, awkward.array.base.AwkwardArray)):
                         if i not in chunks:
                             chunks[i] = []
                         chunks[i].append(x)
@@ -572,7 +568,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
                 pass
 
             else:
-                if isinstance(result, (awkward.util.numpy.ndarray, awkward.array.base.AwkwardArray)):
+                if isinstance(result, (self.numpy.ndarray, awkward.array.base.AwkwardArray)):
                     if None not in chunks:
                         chunks[None] = []
                     chunks[None].append(result)
@@ -580,20 +576,20 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
 
         if out is None:
             if None in chunks:
-                return awkward.array.objects.Methods.maybemixin(types[None], ChunkedArray)(chunks[None])
+                return self.Methods.maybemixin(types[None], ChunkedArray)(chunks[None])
             else:
                 return None
         else:
             for i in range(len(out)):
                 if i in chunks:
-                    out[i] = awkward.array.objects.Methods.maybemixin(types[i], ChunkedArray)(chunks[i])
+                    out[i] = self.Methods.maybemixin(types[i], ChunkedArray)(chunks[i])
             return tuple(out)
 
     def _hasjagged(self):
         for chunkid in range(len(self._chunks)):
             self.knowcounts(until=chunkid)
             if self._counts[chunkid] > 0:
-                return awkward.util._hasjagged(self._chunks[chunkid])
+                return self._util_hasjagged(self._chunks[chunkid])
         else:
             return False
 
@@ -601,7 +597,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
         self.knowcounts()
         self._valid()
 
-        if awkward.util._hasjagged(self):
+        if self._util_hasjagged(self):
             chunks = []
             for chunkid, chunk in enumerate(self._chunks):
                 this = chunk._reduce(ufunc, identity, dtype, regularaxis)
@@ -612,7 +608,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
         out = None
         for chunkid, chunk in enumerate(self._chunks):
             if self._counts[chunkid] > 0:
-                this = awkward.util._reduce(chunk[:self._counts[chunkid]], ufunc, identity, dtype, regularaxis)
+                this = self._util_reduce(chunk[:self._counts[chunkid]], ufunc, identity, dtype, regularaxis)
                 if out is None:
                     out = this
                 else:
@@ -634,11 +630,11 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
             if self._counts[chunkid] > 0:
                 this = chunk[:self._counts[chunkid]]
                 if out is None:
-                    if dtype is None and issubclass(this.dtype.type, (awkward.util.numpy.bool_, awkward.util.numpy.bool)):
-                        dtype = awkward.util.numpy.dtype(type(identity))
+                    if dtype is None and issubclass(this.dtype.type, (self.numpy.bool_, self.numpy.bool)):
+                        dtype = self.numpy.dtype(type(identity))
                     if dtype is None:
                         dtype = this.dtype
-                    out = awkward.util.numpy.empty((sum(self._counts),) + this.shape[1:], dtype=dtype)
+                    out = self.numpy.empty((sum(self._counts),) + this.shape[1:], dtype=dtype)
 
                 newpos = pos + this.shape[0]
                 out[pos:newpos] = this
@@ -647,7 +643,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
         if out is None:
             if dtype is None:
                 dtype = self.DEFAULTTYPE
-            return awkward.util.numpy.array([identity], dtype=dtype)
+            return self.numpy.array([identity], dtype=dtype)
         else:
             return out
 
@@ -710,10 +706,10 @@ class AppendableArray(ChunkedArray):
                 chunks.append(x)
 
         return {"id": ident,
-                "call": ["awkward", self.__class__.__name__],
+                "call": ["awkward", "AppendableArray"],
                 "args": [{"tuple": [{"json": int(x)} for x in self._chunkshape]},
                          {"dtype": awkward.persist.dtype2json(self._dtype)},
-                         {"list": [fill(x, self.__class__.__name__ + ".chunk", prefix, suffix, schemasuffix, storage, compression, **kwargs) for x in chunks]}]}
+                         {"list": [fill(x, "AppendableArray.chunk", prefix, suffix, schemasuffix, storage, compression, **kwargs) for x in chunks]}]}
 
     @property
     def chunkshape(self):
@@ -721,12 +717,12 @@ class AppendableArray(ChunkedArray):
 
     @chunkshape.setter
     def chunkshape(self, value):
-        if isinstance(value, awkward.util.integer) and value > 0:
+        if self._util_isinteger(value) and value > 0:
             self._chunkshape = (value,)
         else:
             try:
                 for x in value:
-                    assert isinstance(x, awkward.util.integer) and x > 0
+                    assert self._util_isinteger(x) and x > 0
             except TypeError:
                 raise TypeError("chunkshape must be a positive integer or a tuple of integers")
             except AssertionError:
@@ -740,7 +736,7 @@ class AppendableArray(ChunkedArray):
 
     @dtype.setter
     def dtype(self, value):
-        self._dtype = awkward.util.numpy.dtype(value)
+        self._dtype = self.numpy.dtype(value)
 
     @property
     def chunks(self):
@@ -752,7 +748,7 @@ class AppendableArray(ChunkedArray):
             iter(value)
         except TypeError:
             raise TypeError("chunks must be iterable")
-        chunks = [awkward.util.toarray(x, self.DEFAULTTYPE, awkward.util.numpy.ndarray) for x in value]
+        chunks = [self._util_toarray(x, self.DEFAULTTYPE, self.numpy.ndarray) for x in value]
         for chunk in chunks:
             if chunk.dtype != self._dtype:
                 raise ValueError("cannot assign chunk with dtype ({0}) to an AppendableArray with dtype ({1})".format(chunk.dtype, self._dtype))
@@ -775,8 +771,7 @@ class AppendableArray(ChunkedArray):
 
     @property
     def offsets(self):
-        import awkward.array.jagged
-        return awkward.array.jagged.counts2offsets(self._counts)
+        return self.JaggedArray.counts2offsets(self._counts)
 
     def _gettype(self, seen):
         return self._dtype
@@ -797,7 +792,7 @@ class AppendableArray(ChunkedArray):
         if len(self._chunks) == 0 or self._counts[-1] == len(self._chunks[-1]):
             self._types.append(None)
             self._counts.append(0)
-            self._chunks.append(awkward.util.numpy.empty(self._chunkshape, dtype=self._dtype))
+            self._chunks.append(self.numpy.empty(self._chunkshape, dtype=self._dtype))
 
         self._chunks[-1][self._counts[-1]] = value
         self._counts[-1] += 1
@@ -807,7 +802,7 @@ class AppendableArray(ChunkedArray):
             if len(self._chunks) == 0 or self._counts[-1] == len(self._chunks[-1]):
                 self._types.append(None)
                 self._counts.append(0)
-                self._chunks.append(awkward.util.numpy.empty(self._chunkshape, dtype=self._dtype))
+                self._chunks.append(self.numpy.empty(self._chunkshape, dtype=self._dtype))
 
             howmany = min(len(values), len(self._chunks[-1]) - self._counts[-1])
             self._chunks[-1][self._counts[-1] : self._counts[-1] + howmany] = values[:howmany]
@@ -821,4 +816,4 @@ class AppendableArray(ChunkedArray):
         chunks = []
         for chunk in self._chunks:
             chunks.append(chunk.astype(dtype))
-        return self.copy(dtype=awkward.util.numpy.dtype(dtype), chunks=chunks)
+        return self.copy(dtype=self.numpy.dtype(dtype), chunks=chunks)
